# 文件和异常
# 函数open()返回一个表示文件的对象，Python将这个对象存储在我们将在后面使用的变量中
# 关键字with在不再需要访问文件后将其关闭，不需要调用close()
# 尽量不要单独使用open()和close()，而是用with open()代替
with open('pi_digits.txt') as file_object:
    # read()读取文件的全部内容，并将其作为一个字符串存储在变量中
    contents = file_object.read()
    print(contents.rstrip())  # 使用rstrip()删除空行

# Python在Windows系统中表示文件路径，一是可以直接使用斜杠（/），比如'./test/1pi_digits.txt'
# 二是使用反斜杠（\）并在字符串前加r声明此字符串不要转义，
# 比如r'C:\Users\Administrator\Documents\GitHub\python-book\test\1pi_digits.txt'
with open('./test/1pi_digits.txt') as file_object:
    print(file_object.read().rstrip())

with open(r'C:\Users\Administrator\Documents\GitHub\python-book\test\1pi_digits.txt') as file_object:
    print(file_object.read().rstrip())

# 逐行读取
with open('pi_digits.txt') as file_object:
    for line in file_object:
        print(line.rstrip())

# 创建一个包含文件各行内容的列表
with open('pi_digits.txt') as file_object:
    lines = file_object.readlines()

for line in lines:
    print(line.rstrip())

# 使用文件的内容
with open('pi_digits.txt') as file_object:
    lines = file_object.readlines()

pi_string = ''
for line in lines:
    pi_string += line.strip()

print(pi_string)
print(len(pi_string))

# 包含一百万位的大型文件
with open('pi_million_digits.txt') as file_object:
    lines = file_object.readlines()

pi_string = ''
for line in lines:
    pi_string += line.strip()

print(pi_string[:52] + '...')
print(len(pi_string))

# 圆周率中包含你的生日吗
birthday = input('please enter your birthday, in the format of yymmdd: ')
if birthday in pi_string:
    print('appear!')
else:
    print('not appear!')

# 写入空文件
# 默认模式r（只读），全部模式参数：https://www.runoob.com/python3/python3-file-methods.html
with open('programming.txt', 'w') as file_object:
    file_object.write('I love programming.\n')
    file_object.write('I love creating new games.\n')

# 异常
# 处理ZeroDivisionError异常
# print(5/0)
# 使用try-except代码块
try:
    print(5 / 0)
except ZeroDivisionError:
    print("can't divide by zero")

# else代码块
print("Give me two numbers, and I'll divide them.")
print("Enter 'q' to quit.")
while True:
    first_number = input("First number: ")
    if first_number == 'q':
        break
    second_number = input("Second number: ")
    if second_number == 'q':
        break
    try:
        answer = int(first_number) / int(second_number)
    except ZeroDivisionError:
        print("You can't divide by 0!")
    else:
        print(answer)

# 处理FileNotFoundError异常
'''
with open('alice.txt') as f_obj:
    contents = f_obj.read()
'''

# 分析文本 & 使用多个文件
title = "Alice's Adventures in Wonderland"
print(title.split())


def count_words(filename):
    try:
        with open(filename) as f_obj:
            contents = f_obj.read()
    except FileNotFoundError:
        print("sorry, the file " + filename + " doesn't exist")
    else:
        words = contents.split()
        num_words = len(words)
        print("the file " + filename + " has about " + str(num_words) + " words")


filename = 'alice.txt'
count_words(filename)

filenames = ['alice.txt', 'siddhartha.txt', 'moby_dict.txt', 'little_women.txt']
for filename in filenames:
    count_words(filename)

# 存储数据
# JSON数据格式并非Python专用，这让你能够将以JSON格式存储的数据与使用其他编程语言的人分享，是一种轻便格式
# JSON（JavaScript Object Notation）格式最初是为JavaScript开发的，但随后成为了一种常见格式，被包括Python在内的众多语言所采用
'''
import json

numbers = [2, 3, 5, 7, 11, 13]
filename = 'numbers.json'
with open(filename, 'w') as f_obj:
    json.dump(numbers, f_obj)  # json.dump()写入数据
'''
import json

filename = 'numbers.json'
with open(filename) as f_obj:
    numbers = json.load(f_obj)  # json.load()读取数据
print(numbers)

# 保存和读取用户生成的数据
import json

filename = 'username.json'
try:
    with open(filename) as f_obj:
        username = json.load(f_obj)
except FileNotFoundError:
    username = input('what is your name? ')
    with open(filename, 'w') as f_obj:
        json.dump(username, f_obj)
        print('we will remember you when you come back, ' + username + '!')
else:
    print('welcome back, ' + username + '!')

# 重构
import json


def get_stored_username():
    """Get stored username if available."""
    filename = 'username.json'
    try:
        with open(filename) as f_obj:
            username = json.load(f_obj)
    except FileNotFoundError:
        return None
    else:
        return username


def get_new_username():
    """Prompt for a new username."""
    username = input("What is your name? ")
    filename = 'username.json'
    with open(filename, 'w') as f_obj:
        json.dump(username, f_obj)
    return username


def greet_user():
    """Greet the user by name."""
    username = get_stored_username()
    if username:
        print("Welcome back, " + username + "!")
    else:
        username = get_new_username()
        print("We'll remember you when you come back, " + username + "!")


greet_user()
